import React, { useState, useEffect, useContext, useRef } from "react";
import Swal from "sweetalert2";
import { useHistory, useLocation } from "react-router-dom";
import _ from "lodash";
import Bowser from "bowser";
import Skeleton, { SkeletonTheme } from "react-loading-skeleton";

import Recorder from "./Recorder";
import {
  HeadingTitle,
  ButtonGeneric,
  SectionLoader,
} from "../_reusables";
import VideoCarousel from "./VideoCarousel";
import InnerHeader from "../Headers/InnerHeader";
import {
  getUserData,
  updateLocalStorage,
  replaceValueSpaceHyphen,
  saveToLocalStorage,
  saveToSessionStorage,
} from "../../utils/Common";
import { DataContext, RecordingContext } from "../../context";
import {
  savePitch,
  updateNextLink,
  getPitchVideos,
  getResumeDetails,
  getReferralLink,
} from "../../api";
import "./_ApplicantPitch.scss";
import Instructions from "./Instructions";
import CamMicTest from "./CamMicTest";
import { JoinMobile, ReferralSection } from "../Dashboard/Sections";
import PitchInstruction from "./PitchInstruction";
// import { InfoIcon } from "../../utils/SvgIcons";
import InfoImg from "../../assets/info.png";

import ChromeIcon from "../../assets/chrome.png";
import FirefoxIcon from "../../assets/firefox.png";
import EdgeIcon from "../../assets/edge.png";
import MicCamImg from "../../assets/mic-test-image.png";
import Steps from "../ResumeQuestionnaire/Steps";

const browser = Bowser.getParser(window.navigator.userAgent);

const ApplicantPitch = () => {
  const [data, setData] = useState({ user_id: 0, picture: null, pitch: [] });
  const [isSaving, setSaving] = useState(false);
  const [hasData, setHasData] = useState(false);
  const [isRetakeCarousel, setRetakeCarousel] = useState(false);
  const [stillImage, setStillImage] = useState(null);
  const [isLoading, setLoading] = useState(false);
  const [isTest, setTest] = useState(false);
  const [isDemo, setDemo] = useState(false);
  const [isInstruction, setInstruction] = useState(true);
  const [isPreVideo, setPreVideo] = useState(true);
  const [resume, setResume] = useState(null);
  const [referral, setReferral] = useState(null);
  const [isContinue, setContinue] = useState(false);
  // const [demoVideo, setDemoVideo] = useState({});
  const instructionVideo =
    "https://media.satelliteteams.com/instructional/pitch/instruction_pitch.mp4";
  const demoVideo =
    "https://media.satelliteteams.com/instructional/pitch/instruction_example.mp4";

  const location = useLocation();

  const dataCheckRef = useRef([]);
  const hasDeleteFromSaved = useRef(false);

  const { userData, currentStep, doneStep, appFlow, setPitchLocation } =
    useContext(DataContext);
  const fromLocal = getUserData();

  const history = useHistory();

  console.log("render parent pitch");

  const handleClick = () => {
    // navigator.mediaDevices
    //   .getUserMedia({ audio: true, video: true })
    //   .then((stream) => {
    //     stream.getTracks().forEach((track) => track.stop());
    //   })
    //   .catch((err) => console.log(err));

    if (data.pitch.length === 0) {
      updateLocalStorage(
        // `/${userData.username || fromLocal.username}/dashboard`
        `/${replaceValueSpaceHyphen(fromLocal.first_name)}/dashboard`
      );

      updateNextLink(
        {
          id: fromLocal.id,
          next_link: `/${replaceValueSpaceHyphen(
            fromLocal.first_name
          )}/dashboard`,
        },
        fromLocal.token
      ).then((resNextLink) => {
        if (resNextLink.data.success) {
          history.push(
            `/${replaceValueSpaceHyphen(fromLocal.first_name)}/dashboard`
          );
        }
      });
      return;
    }
    // if (data.pitch.length < 1) {
    //   Swal.fire({
    //     title: "",
    //     text: "Please record at least 1 recording before you save.",
    //     icon: "error",
    //     confirmButtonText: "Ok",
    //     confirmButtonColor: "#f7685b",
    //   });
    //   return;
    // }

    let newData;

    if (dataCheckRef.current.length > 0) {
      // const hasTellMe = data.pitch.filter(
      //   (item) => item.pitch_question_id === 1
      // );

      // if (hasTellMe.length === 0) {
      //   Swal.fire({
      //     title: "",
      //     text: "Please finish the first recording.",
      //     icon: "error",
      //     confirmButtonText: "Ok",
      //     confirmButtonColor: "#f7685b",
      //   });
      //   return;
      // }

      const vidsBase64 = data.pitch.filter((item) =>
        item.file.includes("data:")
      );

      newData = { ...data, pitch: vidsBase64 };
    } else {
      newData = { ...data, picture: fromLocal.picture ? null : stillImage };
    }

    if (newData.pitch === undefined || newData.pitch.length === 0) {
      history.push(
        `/${replaceValueSpaceHyphen(fromLocal.first_name)}/dashboard`
      );
      return;
    }
    setSaving(true);

    savePitch(newData, userData.token || fromLocal.token)
      .then((res) => {
        if (res.data.success) {
          updateLocalStorage(
            // `/${userData.username || fromLocal.username}/dashboard`
            `/${replaceValueSpaceHyphen(fromLocal.first_name)}/dashboard`
          );
          // if (dataCheckRef.current.length > 0) {
          //   let newPitch = [];
          //   res.data.data.forEach((item) => {
          //     if (item.questions.id === 1) {
          //       newPitch.unshift({
          //         id: item.id,
          //         pitch_question_id: item.questions.id,
          //         file: `https://${item.path || item.path_mp4}`,
          //         pitch_question: item.questions.question,
          //       });
          //     } else {
          //       newPitch.push({
          //         id: item.id,
          //         pitch_question_id: item.questions.id,
          //         file: `https://${item.path || item.path_mp4}`,
          //         pitch_question: item.questions.question,
          //       });
          //     }
          //   });
          //   setData({ user_id: userData.id || fromLocal.id, pitch: newPitch });
          //   setHasData(true);
          //   dataCheckRef.current = newPitch;
          // }

          updateNextLink(
            {
              id: fromLocal.id,
              next_link: `/${replaceValueSpaceHyphen(
                fromLocal.first_name
              )}/dashboard`,
            },
            fromLocal.token
          )
            .then((resNextLink) => {
              if (resNextLink.data.success) {
                Swal.fire({
                  title: "",
                  text: "Pitch recordings saved.",
                  icon: "success",
                  confirmButtonText: "Ok",
                  confirmButtonColor: "#55428A",
                }).then(() => {
                  history.push(
                    `/${replaceValueSpaceHyphen(
                      fromLocal.first_name
                    )}/dashboard`
                  );
                  if (dataCheckRef.current.length === 0) {
                    saveToLocalStorage({
                      ...fromLocal,
                      picture: fromLocal.picture
                        ? fromLocal.picture
                        : stillImage,
                      next_link: `/${replaceValueSpaceHyphen(
                        fromLocal.first_name
                      )}/dashboard`,
                    });
                    saveToSessionStorage({
                      ...fromLocal,
                      picture: fromLocal.picture
                        ? fromLocal.picture
                        : stillImage,
                      next_link: `/${replaceValueSpaceHyphen(
                        fromLocal.first_name
                      )}/dashboard`,
                    });

                    let newPitch = [];
                    res.data.data.forEach((item) => {
                      if (item.questions.id === 1) {
                        newPitch.unshift({
                          id: item.id,
                          pitch_question_id: item.questions.id,
                          file: `https://${item.path || item.path_mp4}`,
                          pitch_question: item.questions.question,
                        });
                      } else {
                        newPitch.push({
                          id: item.id,
                          pitch_question_id: item.questions.id,
                          file: `https://${item.path || item.path_mp4}`,
                          pitch_question: item.questions.question,
                        });
                      }
                    });
                    setData({
                      user_id: userData.id || fromLocal.id,
                      pitch: newPitch,
                    });
                    setHasData(true);
                    dataCheckRef.current = newPitch;
                  }
                });
              }
            })
            .finally(() => setSaving(false));
        }
      })
      .catch((err) => {
        setSaving(false);

        if (err.response.status === 422 || err.response.status === 401) {
          // Swal.fire({
          //   title: "Connectivity",
          //   text: "Something went wrong, kindly check if you're experiencing unstable internet connection",
          //   icon: "warning",
          //   confirmButtonText: "Ok",
          //   confirmButtonColor: "#55428A",
          // });
          Swal.fire({
            title: "Unable to process your recordings",
            text: err.response.message,
            icon: "warning",
            confirmButtonText: "Ok",
            confirmButtonColor: "#55428A",
          });
        } else {
          console.log(err.response.status);
        }
      });
  };

  useEffect(() => {
    const controller = new AbortController();
    if (browser.getBrowserName() === "Safari") {
      Swal.fire({
        customClass: {
          htmlContainer: "pitch-browser-compatibility",
        },
        html: `<p>Opps! You're using a web browser that has compatibility issues.</p>
          <p>To have a better experience, please try using different browser like:</p>
          <div>
            <div>
              <img src="${ChromeIcon}" height="48" width="48" alt="chrome" />
              <img src="${FirefoxIcon}" height="48" width="48" alt="firefox" />
              <img src="${EdgeIcon}" height="48" width="48" alt="edge" />
            </div>
          </div>
        `,
        icon: "warning",
        confirmButtonColor: "#7964b5",
        confirmButtonText: "Done",
      });
    }

    if (_.isEmpty(userData) && fromLocal === null) {
      history.push("/login");
      return;
    } else if (
      fromLocal.next_link === "" ||
      fromLocal.next_link === null ||
      fromLocal.next_link === undefined
    ) {
      history.push(`/${replaceValueSpaceHyphen(fromLocal.first_name)}/resume`);
      return;
    }

    if (process.env.REACT_APP_MAINTENANCE === "1") {
      history.push("/maintenance");
      return;
    }

    if (process.env.REACT_APP_LAUNCHING === "1") {
      history.push("/coming-soon");
      return;
    }

    if (data.pitch.length === 0) {
      setLoading(true);
      getPitchVideos(fromLocal?.id, fromLocal?.token, controller)
        .then((res) => {
          if (res.status === 200) {
            if (res.data.success) {
              getResumeDetails(fromLocal?.id, fromLocal?.token, controller)
                .then((resResume) => {
                  if (resResume.data.success) {
                    getReferralLink(fromLocal?.token, controller)
                      .then((referralRes) => {
                        if (referralRes.data.success) {
                          if (res.data.data.length > 0) {
                            let newPitch = [];
                            res.data.data.forEach((item) => {
                              if (item.questions.id === 1) {
                                newPitch.unshift({
                                  id: item.id,
                                  pitch_question_id: item.questions.id,
                                  // file: `https://${item.path}`,
                                  file: item.path || item.path_mp4,
                                  pitch_question: item.questions.question,
                                });
                              } else {
                                newPitch.push({
                                  id: item.id,
                                  pitch_question_id: item.questions.id,
                                  file: item.path || item.path_mp4,
                                  pitch_question: item.questions.question,
                                });
                              }
                            });
                            setData({
                              user_id: userData.id || fromLocal.id,
                              pitch: newPitch,
                            });
                            setHasData(true);
                            dataCheckRef.current = newPitch;
                            setPreVideo(false);

                            setResume(resResume.data.data);
                            setReferral(referralRes.data.data);
                            setPitchLocation(true);
                            setLoading(false);

                            if (newPitch.length > 0) {
                              const tellMe = newPitch.filter(
                                (item) => item.pitch_question_id === 1
                              );

                              if (tellMe.length === 0) {
                                hasDeleteFromSaved.current = true;
                                setRetakeCarousel(true);
                              }
                            }
                          } else {
                            setResume(resResume.data.data);
                            setReferral(referralRes.data.data);
                            setPitchLocation(true);
                            setLoading(false);
                          }
                        }
                      })
                      .catch((err) => {
                        if (err.response.status === 401) {
                          Swal.fire({
                            title: "Error!",
                            text: err.response.data.message,
                            icon: "error",
                            confirmButtonText: "Close",
                            confirmButtonColor: "#55428A",
                          });
                          return;
                        }

                        if (err.response.status === 500) {
                          console.log(err.response.data.message);
                        }
                      });
                  }
                })
                .catch((err) => {
                  if (err.response.status === 401) {
                    Swal.fire({
                      title: "Error!",
                      text: err.response.data.message,
                      icon: "error",
                      confirmButtonText: "Close",
                      confirmButtonColor: "#55428A",
                    });
                    return;
                  }

                  if (err.response.status === 500) {
                    console.log(err.response.data.message);
                  }
                });
            }
          }
        })
        .catch((err) => {
          if (err.response.status === 401) {
            Swal.fire({
              title: "Error!",
              text: err.response.data.message,
              icon: "error",
              confirmButtonText: "Close",
              confirmButtonColor: "#55428A",
            });
            return;
          }

          if (err.response.status === 500) {
            console.log(err.response.data.message);
          }
        });
    }

    return () => controller.abort();
  }, []);

  return (
    <RecordingContext.Provider
      value={{
        data,
        setData,
        isSaving,
        setSaving,
        hasData,
        setHasData,
        isRetakeCarousel,
        setRetakeCarousel,
        hasDeleteFromSaved,
        dataCheckRef,
        stillImage,
        setStillImage,
        isLoading,
        setTest,
        setDemo,
        demoVideo,
        setInstruction,
        instructionVideo,
        isInstruction,
        setPreVideo,
        isDemo,
        handleClick,
        isContinue,
        setContinue,
      }}
    >
      {/* {fromLocal.next_link.includes("/pitch") ? <Header /> : <InnerHeader />} */}
      {/* <InnerHeader /> */}
      {isTest && <CamMicTest setTest={setTest} />}
      <div className="container">
        <div className="pitch-main-container">
          <div className="grid-white-box-container">
            <Instructions loading={isLoading} />
          </div>
          <div className="grid-white-box-container">
            <JoinMobile loading={isLoading} />
          </div>
          <div className="grid-white-box-container">
            <ReferralSection loading={isLoading} referralLink={referral} />
          </div>
          <div className="grid-white-box-container">
            <div>
              <HeadingTitle
                text={isPreVideo ? "Video guide" : "Recorder"}
                isPrimaryColor
                loading={isLoading}
              >
                {isLoading ? (
                  <Skeleton width={300} height={14} />
                ) : (
                  <p className="pitch-role">
                    Job role:{" "}
                    <span>
                      {resume?.job_role[0]?.position || "No position"}
                    </span>
                  </p>
                )}
              </HeadingTitle>
              {isLoading ? (
                <>
                  <Skeleton width={"100%"} height={350} />
                  <div style={{ textAlign: "center", marginTop: 20 }}>
                    <Skeleton width={300} height={40} />
                  </div>
                </>
              ) : (
                <>
                  {isPreVideo ? <PitchInstruction /> : <Recorder />}
                  <div className="test-mic-container">
                    {/* <ButtonGeneric
                  text="Test Microphone and Webcam"
                  size="small"
                  color={"yellow"}
                  hasOutline={true}
                  event={() => setTest(true)}
                /> */}
                    <button type="button" onClick={() => setTest(true)}>
                      <p>Tap to test the camera and microphone.</p>
                      {/* <img src={MicCamImg} alt="mic" /> */}
                    </button>
                  </div>
                </>
              )}
            </div>
          </div>
          <div className="grid-white-box-container">
            <HeadingTitle
              text="Your recorded pitch"
              isPrimaryColor
              loading={isLoading}
            />
            {isLoading ? (
              <>
                <div
                  style={{
                    display: "grid",
                    gridTemplateColumns: "auto 130px",
                    gap: 20,
                  }}
                >
                  <Skeleton width={"100%"} height={350} />
                  <div
                    style={{
                      display: "flex",
                      flexDirection: "column",
                      gap: 20,
                    }}
                  >
                    <SkeletonTheme width={120} height={70}>
                      <Skeleton />
                      <Skeleton />
                      <Skeleton />
                      <Skeleton />
                    </SkeletonTheme>
                  </div>
                </div>
                <p style={{ marginTop: 20 }}>
                  <Skeleton count={2} />
                </p>
              </>
            ) : (
              <VideoCarousel data={data} setData={setData} />
            )}
          </div>
          <div>
            {isLoading ? (
              <Skeleton width={"100%"} height={40} />
            ) : (
              <div className="temp">
                <div className="note-reminder">
                  <img src={InfoImg} alt="info" />
                  <p>
                    <b>Note:</b> If this pitch recording does not work on this
                    browser, please try another browser.
                  </p>
                </div>
              </div>
            )}
          </div>
        </div>
        <div className="done-btn-container">
          {isLoading ? (
            <Skeleton width={150} height={50} />
          ) : (
            <ButtonGeneric
              disabled={dataCheckRef.current.length > 3 ? true : isSaving}
              text={isSaving ? "Saving..." : "Done"}
              hasOutline={true}
              event={handleClick}
              loading={isSaving}
            />
          )}
        </div>
        {/* <div className="white-box-container pitch-container">
          <HeadingTitle text="Creating your pitch video" />
          <div className="pitch-sub-container">
            {isLoading ? (
              <SectionLoader color1={"#463672"} color2={"#fff"} />
            ) : (
              <>
                <Instructions />
                <div className="pitch-sub-second-row">
                  {isPreVideo ? <PitchInstruction /> : <Recorder />}
                  <VideoCarousel data={data} setData={setData} />
                </div>
                <div className="temp">
                  <div className="note-reminder">
                    {InfoIcon("#f53a29")}
                    <p>
                      Note: If this pitch recording does not work on this
                      browser, please try another browser.
                    </p>
                  </div>
                </div>
                <div className="done-btn-container">
                  <ButtonGeneric
                    disabled={dataCheckRef.current.length > 3 ? true : isSaving}
                    text={isSaving ? "Saving..." : "Done"}
                    hasOutline={true}
                    event={handleClick}
                    loading={isSaving}
                  />
                </div>
              </>
            )}
          </div>
        </div> */}
      </div>
      {fromLocal?.next_link?.includes("/pitch") && (
        <div className="global-steps-container">
          {appFlow.map((step) => (
            <Steps
              key={step.index}
              text={step.text}
              index={step.index}
              isCurrent={currentStep}
              isDone={doneStep}
            />
          ))}
        </div>
      )}
    </RecordingContext.Provider>
  );
};

export default ApplicantPitch;
