import React, { useRef, useState, useContext } from "react";

import { ButtonGeneric } from "../_reusables";
// import VideoDemo from "../../assets/App Instructional Video.mp4";
import { DataContext, RecordingContext } from "../../context";
import { getUserData } from "../../utils/Common";

import "./_PitchInstruction.scss";

const PitchInstruction = () => {
  const [isPlay, setPlay] = useState(false);
  const [videoDone, setVideoDone] = useState(false);
  const [instructionDone, setInstructionDone] = useState(false);
  const videoRef = useRef(null);

  const { userData } = useContext(DataContext);
  const {
    setDemo,
    demoVideo,
    instructionVideo,
    isInstruction,
    setPreVideo,
    isDemo,
    setInstruction,
  } = useContext(RecordingContext);
  const fromLocal = getUserData();

  const handlePlay = () => {
    setPlay(true);
    videoRef.current.play();
  };

  const handleContinue = () => {
    setPreVideo(false);
  };

  const handlePlayAgain = () => {
    setPlay(false);
    setVideoDone(false);
    if (isInstruction) {
      setInstruction(false);
      setDemo(true);
    } else if (isDemo) {
      setDemo(false);
      setInstruction(true);
    }
  };

  const onVideoEnded = () => {
    setVideoDone(true);
    setPlay(false);
  };

  return (
    <div>
      <div className="demo-preview-container">
        <div className="demo-preview">
          {videoDone && (
            <>
              <div className="demo-done-container">
                <p>
                  When you're ready, click continue to start your pitch or click
                  on the play demo button to see a sample pitch recording.
                </p>
                <p>Best of luck, {fromLocal.first_name}!</p>
                <div>
                  <ButtonGeneric
                    text="Continue"
                    size="extraSmall"
                    hasOutline={true}
                    event={handleContinue}
                  />
                  <ButtonGeneric
                    text={isInstruction ? "Play demo" : "Play instruction"}
                    size="extraSmall"
                    color="blue"
                    hasOutline={true}
                    event={handlePlayAgain}
                  />
                </div>
              </div>
              <div className="demo-overlay"></div>
            </>
          )}
          {!isPlay && !videoDone && (
            <div className="demo-play-btn" onClick={handlePlay}>
              Play
            </div>
          )}
          {demoVideo && (
            <video
              src={isInstruction ? instructionVideo : demoVideo}
              type="video/mp4"
              ref={videoRef}
              controls={isPlay}
              controlsList="nodownload"
              onEnded={onVideoEnded}
              height={370}
            />
          )}
        </div>
      </div>
    </div>
  );
};

export default PitchInstruction;
