import React, {
  useEffect,
  useState,
  useContext,
  useRef,
  useCallback,
} from "react";
import { useRecordWebcam } from "react-record-webcam";
import _ from "lodash";
import { useHistory, useLocation } from "react-router-dom";
import captureVideoFrame from "capture-video-frame";
import Bowser from "bowser";

import Timer from "./Timer";
import { DataContext, RecordingContext } from "../../context";
import { getUserData } from "../../utils/Common";
import { ButtonGeneric } from "../_reusables";
import NextStep from "./NextStep";

import { getPitchQuestion, savePitch } from "../../api";

import "./_Recorder.scss";

const browser = Bowser.getParser(window.navigator.userAgent);

const Recorder = () => {
  const {
    data,
    setData,
    hasData,
    isRetakeCarousel,
    setRetakeCarousel,
    setHasData,
    dataCheckRef,
    stillImage,
    setStillImage,
    setTest,
    isTellMe,
    isContinue,
    setContinue,
  } = useContext(RecordingContext);

  console.log("render recorder");

  const history = useHistory();

  const [initLength, setInitLength] = useState(60);
  const [countdown, setCountDown] = useState(10);
  const [isRecording, setRecording] = useState(false);
  const [recordSteps, setRecordSteps] = useState(hasData ? 5 : 1);
  const [questions, setQuestions] = useState([]);
  const [isRetake, setRetake] = useState(false);
  const [isNextStep, setNextStep] = useState(false);

  const stepReserveRef = useRef(0);

  const { userData } = useContext(DataContext);
  const fromLocal = getUserData();

  const recordWebcam = useRecordWebcam({
    recordingLength: initLength,
    fileType: "webm",
    width: 500,
    height: 500,
    aspectRatio: 1.77,
    codec: { audio: "aac" | "opus", video: "vp8" },
  });

  const startRecord = () => {
    if (isRetakeCarousel) {
      setRetakeCarousel(false);
    }
    setRecording(true);
    recordWebcam.start();
  };

  const stopRecord = () => {
    recordWebcam.stop();
    recordWebcam.close();
    setInitLength(0);
  };

  const nexStep = () => {
    setNextStep(true);
  };

  const startOptional = () => {
    setInitLength(30);
    if (isRetake) setRetake(false);
    if (isNextStep) setNextStep(false);
    recordWebcam.retake();
  };

  const continueOptional = () => {
    setInitLength(30);
    setContinue(false);
    recordWebcam.retake();
  };

  const retakeRecording = () => {
    setInitLength(60);
    setRetake(true);
    if (!isRetakeCarousel) {
      setRecordSteps((prevState) => prevState - 1);
    }
    recordWebcam.retake();
  };

  const saveData = useCallback(
    (retake) => {
      setTimeout(async () => {
        const blob = await recordWebcam.getRecording();

        const file = new File([blob], `pitch-${recordSteps}.webm`, {
          type: "video/webm",
        });

        const reader = new FileReader();

        reader.readAsDataURL(file);

        reader.onload = (e) => {
          const videoFile = e.target.result;

          const pitchData = {
            pitch_question_id:
              recordSteps === 1 ? 1 : questions[recordSteps - 1]?.id,
            pitch_question: questions[recordSteps - 1]?.question,
            file: videoFile,
          };
          const formData = {
            user_id: userData.id || fromLocal.id,
            pitch: data.pitch,
          };
          if (retake || isRetakeCarousel) {
            console.log("retake");
            const pitchArrWithoutFirst = formData.pitch.filter(
              (item) => item.pitch_question_id !== 1
            );

            console.log(pitchArrWithoutFirst);
            const newPitchArr = [pitchData, ...pitchArrWithoutFirst];
            setData({ ...formData, pitch: newPitchArr });
            setRecording(false);
            setRetake(false);

            if (stepReserveRef.current !== 0) {
              console.log("run !== 0");
              setRecordSteps(5);
              stepReserveRef.current = 1;
              dataCheckRef.current = newPitchArr;
            } else {
              if (dataCheckRef.current.length === 3) {
                setRecordSteps(5);
              } else {
                setRecordSteps((prevState) => prevState + 1);
              }
            }
          } else {
            console.log(pitchData);
            if (stepReserveRef.current !== 0) {
              setRecordSteps(stepReserveRef.current);
              stepReserveRef.current = 0;
              const pitchArr = [pitchData, ...formData.pitch];
              setData({ ...formData, pitch: pitchArr });
              if (data.pitch.length === 1) {
                console.log("pitch 2");
                setRecordSteps(3);
              } else if (data.pitch.length === 2) {
                setRecordSteps(4);
              } else {
                setRecordSteps((prevState) => prevState + 1);
              }
            } else {
              let pitchArr = [...formData.pitch];

              if (pitchData.pitch_question_id === 1) {
                pitchArr = [pitchData, ...formData.pitch];
              } else {
                pitchArr = [...formData.pitch, pitchData];
              }

              setData({ ...formData, pitch: pitchArr });
              if (data.pitch.length === 1) {
                setRecordSteps(3);
              } else if (data.pitch.length === 2) {
                setRecordSteps(4);
              } else {
                setRecordSteps((prevState) => prevState + 1);
              }
            }
            setRecording(false);
          }

          recordWebcam.close();
        };
      }, 1000);
    },
    [recordWebcam]
  );

  useEffect(() => {
    if (initLength === 0 && !isRetake && !isRetakeCarousel) {
      saveData(false);
    }

    if (initLength === 0 && isRetake && !isRetakeCarousel) {
      saveData(true);
    }

    // code for frame capture
    if (recordSteps === 1 && dataCheckRef.current.length === 0) {
      if (stillImage === null && isRecording && initLength === 58) {
        const frame = captureVideoFrame(recordWebcam.webcamRef.current, "png");

        setStillImage(frame.dataUri);
      }
    }
  }, [initLength]);

  useEffect(() => {
    const controller = new AbortController();
    if (
      (hasData && dataCheckRef.current.length === 4) ||
      (!isRetakeCarousel && dataCheckRef.current.length === 4)
    ) {
      setRecordSteps(5);
      setHasData(false);
    }

    if (hasData && dataCheckRef.current.length === 1 && !isRecording) {
      const tellMe = data.pitch.filter((item) => item.pitch_question_id === 1);

      if (tellMe.length === 1) {
        setRecordSteps(2);
        setHasData(false);
        setContinue(true);
        console.log("has, 1 length");
      } else {
        setRecordSteps(1);
        stepReserveRef.current = 2;
        console.log("nope, 1 length");
      }
    }

    if (hasData && dataCheckRef.current.length === 2 && !isRecording) {
      const tellMe = data.pitch.filter((item) => item.pitch_question_id === 1);

      if (tellMe.length === 1) {
        setRecordSteps(3);
        setHasData(false);
        setContinue(true);
        console.log("has, 2 length");
      } else {
        setRecordSteps(1);
        stepReserveRef.current = 3;
        console.log("nope, 2 length");
      }
    }

    if (hasData && dataCheckRef.current.length === 3 && !isRecording) {
      const tellMe = data.pitch.filter((item) => item.pitch_question_id === 1);

      if (tellMe.length === 1) {
        setRecordSteps(4);
        setHasData(false);
        setContinue(true);
        console.log("has, 3 length");
      } else {
        setRecordSteps(1);
        stepReserveRef.current = 4;
        console.log("has, 3 length");
      }
    }

    if (isRetakeCarousel && recordSteps !== 1) {
      console.log("yes");
      stepReserveRef.current = recordSteps;
      setRecordSteps(1);
      setInitLength(60);
      if (!hasData) {
        stepReserveRef.current = 0;
      }
      // retakeRecording()
    }

    if (questions.length === 0) {
      getPitchQuestion(userData.token || fromLocal.token, controller)
        .then((res) => {
          if (res.data.success) {
            setQuestions(res.data.data);
            // if (hasData && dataCheckRef.current.length === 1) {
            //   setRecordSteps(2);
            //   setInitLength(30);
            // }
          }
        })
        .catch((err) => console.log(err));
    }

    return () => {
      console.log("unmount");
      recordWebcam.close();
      controller.abort();
    };
  }, [questions.length, hasData, isRetakeCarousel]);

  return (
    <div>
      <div className="webcam-container">
        {initLength === 0 && !isRecording && recordSteps >= 2 && (
          <div className="btn-container">
            {!isNextStep && recordSteps <= 2 && (
              <>
                <span className="normal-btn" onClick={retakeRecording}>
                  Retake recording
                </span>
                {data.pitch.length === 1 && (
                  <span className="normal-btn" onClick={nexStep}>
                    Continue to the next step
                  </span>
                )}
              </>
            )}
            {recordSteps > 2 && recordSteps < 5 && !isNextStep && (
              <span className="normal-btn" onClick={startOptional}>
                Next question
              </span>
            )}
          </div>
        )}
        {isContinue && (
          <div className="btn-container">
            <span className="normal-btn" onClick={continueOptional}>
              Continue
            </span>
          </div>
        )}
        {initLength > 0 && !isRecording && recordWebcam.status === "OPEN" && (
          <div className="btn-container">
            <span className="record-btn" onClick={startRecord}>
              Start
            </span>
          </div>
        )}
        {recordSteps === 1 && !isRecording && recordWebcam.status === "CLOSED" && (
          <div className="btn-container">
            <span className="normal-btn" onClick={recordWebcam.open}>
              Start your pitch
            </span>
          </div>
        )}

        {isRecording && (
          <>
            {/* {recordSteps !== 1 && !isRetake && ( */}
            <div className="btn-container">
              <span className="stop-btn" onClick={stopRecord}>
                Stop
              </span>
            </div>
            {/* )} */}
            <Timer initLength={initLength} setInitLength={setInitLength} />
          </>
        )}
        {isNextStep && <NextStep startOptional={startOptional} />}
        <div className="cam-container">
          {!isRecording && <div className="video-overlay"></div>}
          <video ref={recordWebcam.webcamRef} autoPlay muted height={370} />
        </div>
      </div>
      {recordSteps < 5 &&
        questions &&
        !isNextStep &&
        initLength !== 0 &&
        !isContinue && (
          <div className="label-cta-container">
            {recordSteps > 1 && recordSteps < 5 && (
              <p>{questions[recordSteps - 1]?.question}</p>
            )}
          </div>
        )}
      {recordSteps >= 5 && (
        <div className="label-cta-container">
          <p>Recording done.</p>
        </div>
      )}
      {/* <div className="label-cta-container"> */}
      {/* {recordSteps < 5 && questions && (
          <p>
            {recordSteps === 1
              ? "Tell us about yourself?"
              : questions[recordSteps - 1]?.question}
          </p>
        )} */}
      {/* {recordSteps === 5 && <p>Recording Done.</p>} */}
      {/* </div> */}
    </div>
  );
};

export default Recorder;
