import React, { useContext, useState } from "react";
import _ from "lodash";
import Bowser from "bowser";

import VideoPreview from "./VideoPreview";
import VideoThumb from "./VideoThumb";
import { HeadingTitle, SectionLoader } from "../_reusables";
import { RecordingContext } from "../../context";
import { videoPitchIcon, vhsIcon, VideoCheck } from "../../utils/SvgIcons";

import "./_VideoCarousel.scss";

const carBtnLeft = require("../../assets/pitch-slider-btn-left.png");
const carBtnRight = require("../../assets/pitch-slider-btn-right.png");

const browser = Bowser.getParser(window.navigator.userAgent);

const VideoCarousel = () => {
  const { data, isLoading } = useContext(RecordingContext);

  const [videoSelected, setVideoSelected] = useState({});

  const settings = {
    infinite: false,
    speed: 500,
    slidesToShow: 2,
    slidesToScroll: 2,
    lazyLoad: true,
    nextArrow: <img src={carBtnRight} alt="next" />,
    prevArrow: <img src={carBtnLeft} alt="prev" />,
    mobileFirst: true,
    swipeToSlide: true,
    responsive: [
      {
        breakpoint: 900,
        settings: {
          slidesToShow: 1,
          slidesToScroll: 1,
          // arrows: false
        },
      },
    ],
  };

  const handleVideoSelect = (questionId) => {
    const selected = data.pitch.filter(
      (item) => item.pitch_question_id === questionId
    );
    setVideoSelected(selected[0]);
  };

  return (
    <div className="carousel-container">
      {/* <HeadingTitle text="Your Recorded Videos" /> */}
      <>
        {!_.isEmpty(data.pitch) ? (
          browser.getBrowserName() === "Safari" ? (
            <div className="safari-pitch-issue">
              <div>
                {VideoCheck("#7964b5")}
                {data.pitch.length >= 4 ? (
                  <>
                    <p>You completed all the recordings.</p>
                  </>
                ) : (
                  <>
                    <p>Your recording has been processed.</p>
                    <p>{`Please proceed to next recording ${
                      data.pitch.length + 1
                    } or Click Done Button to save your recordings.`}</p>
                  </>
                )}
              </div>
            </div>
          ) : (
            <div className="recordings-container">
              <div>
                <div className="preview-container">
                  <div>
                    {!_.isEmpty(videoSelected) ? (
                      <VideoPreview
                        pitch_question_id={videoSelected.pitch_question_id}
                        pitch_question={videoSelected.pitch_question}
                        url={videoSelected.file}
                        noMargin={true}
                      />
                    ) : (
                      <div className="no-selected-recording">
                        <div>
                          {vhsIcon("#cfd5de")}
                          <p>Select a recording to preview.</p>
                        </div>
                      </div>
                    )}
                  </div>
                </div>
              </div>
              <div className="video-list-container">
                {!_.isEmpty(data.pitch) &&
                  data.pitch.map((item, i) => (
                    <VideoThumb
                      key={i}
                      url={item.file}
                      clickEvt={() => handleVideoSelect(item.pitch_question_id)}
                      pitch_question_id={item.pitch_question_id}
                      hasCta={item.pitch_question_id === 1 ? true : false}
                      videoSelected={videoSelected}
                      setVideoSelected={setVideoSelected}
                    />
                  ))}
              </div>
              <div className="preview-question-container">
                <p>{videoSelected.pitch_question}</p>
              </div>
            </div>
          )
        ) : (
          <div className="empty-records-list">
            <div>
              {videoPitchIcon("#dee2e8")}
              <p>No pitch recordings</p>
            </div>
          </div>
        )}
      </>
      {/* {isLoading ? (
        <SectionLoader color1={"#463672"} color2={"#fff"} />
      ) : (
        <>
          {!_.isEmpty(data.pitch) ? (
            browser.getBrowserName() === "Safari" ? (
              <div className="safari-pitch-issue">
                <div>
                  {VideoCheck("#7964b5")}
                  {data.pitch.length >= 4 ? (
                    <>
                      <p>You completed all the recordings.</p>
                    </>
                  ) : (
                    <>
                      <p>Your recording has been processed.</p>
                      <p>{`Please proceed to next recording ${
                        data.pitch.length + 1
                      } or Click Done Button to save your recordings.`}</p>
                    </>
                  )}
                </div>
              </div>
            ) : (
              <div className="recordings-container">
                <div className="preview-container">
                  <div>
                    {!_.isEmpty(videoSelected) ? (
                      <VideoPreview
                        pitch_question_id={videoSelected.pitch_question_id}
                        pitch_question={videoSelected.pitch_question}
                        url={videoSelected.file}
                        noMargin={true}
                      />
                    ) : (
                      <div className="no-selected-recording">
                        <div>
                          {vhsIcon("#cfd5de")}
                          <p>Select a recording to preview.</p>
                        </div>
                      </div>
                    )}
                  </div>
                </div>
                <div className="preview-question-container">
                  <p>{videoSelected.pitch_question}</p>
                </div>
                <div className="video-list-container">
                  {!_.isEmpty(data.pitch) &&
                    data.pitch.map((item, i) => (
                      <VideoThumb
                        key={i}
                        url={item.file}
                        clickEvt={() =>
                          handleVideoSelect(item.pitch_question_id)
                        }
                        pitch_question_id={item.pitch_question_id}
                        hasCta={item.pitch_question_id === 1 ? true : false}
                        videoSelected={videoSelected}
                        setVideoSelected={setVideoSelected}
                      />
                    ))}
                </div>
              </div>
            )
          ) : (
            <div className="empty-records-list">
              <div>
                {videoPitchIcon("#dee2e8")}
                <p>No Pitch Recordings</p>
              </div>
            </div>
          )}
        </>
      )} */}
    </div>
  );
};

export default VideoCarousel;
