import React, { useState, useContext } from "react";
import Swal from "sweetalert2";

// import { ButtonGeneric } from "../_reusables";
import { RecordingContext, DataContext } from "../../context";
import { getUserData } from "../../utils/Common";
import { TrashMediumIcon } from "../../utils/SvgIcons";
import { deletePitchRecording } from "../../api";
import "./_VideoThumb.scss";

const VideoThumb = ({
  url,
  clickEvt,
  hasCta,
  pitch_question_id,
  videoSelected,
  setVideoSelected,
}) => {
  const [isDelete, setDelete] = useState(false);

  const {
    data,
    setData,
    setRetakeCarousel,
    hasDeleteFromSaved,
    dataCheckRef,
    isContinue,
    setContinue,
  } = useContext(RecordingContext);
  const userData = useContext(DataContext);
  const fromLocal = getUserData();

  const clearPreview = () => {
    if (pitch_question_id === videoSelected.pitch_question_id) {
      setVideoSelected({});
    }
  };

  const handleDelete = () => {
    setDelete(true);
    let file = data.pitch.filter(
      (item) => item.pitch_question_id === pitch_question_id
    );
    const newData = data.pitch.filter(
      (item) => item.pitch_question_id !== pitch_question_id
    );

    const hasId = Object.prototype.hasOwnProperty.call(file[0], "id");

    if (hasId) {
      deletePitchRecording(file[0].id, userData.token || fromLocal.token)
        .then((res) => {
          if (res.data.success) {
            Swal.fire({
              title: "",
              text: "Video deleted",
              icon: "info",
              confirmButtonColor: "#7964b5",
              confirmButtonText: "Done",
            });
            setDelete(false);
            setData({ ...data, pitch: newData });
            hasDeleteFromSaved.current = true;
            dataCheckRef.current = newData;
            setRetakeCarousel(true);
            if (isContinue) {
              setContinue(false);
            }
            clearPreview();
          }
        })
        .catch((err) => {
          setDelete(false);
          if (err.response.status === 401) {
            Swal.fire({
              title: "Error!",
              text: err.response.data.message,
              icon: "error",
              confirmButtonColor: "#55428A",
              confirmButtonText: "Close",
            });

            return;
          }

          if (err.response.status === 422) {
            Swal.fire({
              title: "Error!",
              text: err.response.data.message,
              icon: "error",
              confirmButtonColor: "#55428A",
              confirmButtonText: "Close",
            });

            return;
          }

          if (err.response.status === 500) {
            console.log(err.response.data.message);
          }
        });
    } else {
      setDelete(false);
      setData({ ...data, pitch: newData });
      hasDeleteFromSaved.current = false;
      if (dataCheckRef.current.length > 0) {
        dataCheckRef.current = newData;
      }

      if (isContinue) {
        setContinue(false);
      }

      setRetakeCarousel(true);
      clearPreview();
    }
  };
  const onMouseOver = (e) => {
    e.target.play();
  };

  const omMouseOut = (e) => {
    e.target.pause();
    e.target.currentTime = 0;
  };

  return (
    <div className={`video-thumb-container`}>
      <div
        className={`${
          videoSelected.pitch_question_id === pitch_question_id
            ? "video-thumb-playing"
            : ""
        }`}
      >
        {hasCta && (
          <button onClick={handleDelete} type="button">
            {TrashMediumIcon("#fff")}
          </button>
        )}
        <video
          src={url}
          controls={false}
          onClick={clickEvt}
          onMouseOver={onMouseOver}
          onMouseOut={omMouseOut}
          muted={true}
          playsInline
          // type="video/webm"
        />
      </div>
    </div>
  );
};

export default VideoThumb;
