import React, { useEffect, useState } from "react";
import Swal from "sweetalert2";

import { ButtonGeneric } from "../_reusables";
import { getTextSpeechCategoryScore } from "../../api";
import { getUserData } from "../../utils/Common";
import "./_CategoryCard.scss";

const CategoryCard = ({
  category,
  index,
  userTextOutput,
  handleShowAssesment,
}) => {
  const [finishText, setFinishText] = useState(0);
  const [showScore, setShowScore] = useState(false);
  const [cat, setCat] = useState(null);

  const fromLocal = getUserData();

  const handleShowScore = async () => {
    console.log("test");
    try {
      setShowScore(true);
      const scoreRes = await getTextSpeechCategoryScore(
        { user_id: fromLocal.id, category_id: cat.id },
        fromLocal.token
      );

      const {
        quality_score_average,
        fluency_score_average,
        pte_score_average,
        ielts_score_average,
      } = scoreRes.data.data;

      setCat((prev) => ({
        ...prev,
        score: {
          quality_score_average,
          fluency_score_average,
          pte_score_average,
          ielts_score_average,
        },
      }));
    } catch (err) {
      if (err.response.status === 401 || err.response.status === 402) {
        Swal.fire({
          title: "Error!",
          text: err.response.data.message,
          icon: "error",
          confirmButtonText: "Close",
          confirmButtonColor: "#55428A",
        });
        return;
      }

      if (err.response.status === 500) {
        console.log(err.response.status);
      }
    } finally {
      setShowScore(false);
    }
  };

  useEffect(() => {
    console.log(category?.score);
    if (!category?.score) {
      const current = userTextOutput.current.find(
        (item) => item.category === category.category
      );

      if (current) {
        if (current?.text?.length <= category.text.length) {
          setFinishText(current.text.length);
        }
      }
    }

    setCat(category);
  }, []);

  return (
    <div className="assesment-card-container">
      <p>
        {cat?.category === "Pronounciation" ? "Pronunciation" : cat?.category}
      </p>
      {cat?.score ? (
        <>
          <div className="assesment-card-scores">
            <p>Total score:</p>
            <div className="card-score">
              <p>
                <span>
                  {Math.round(parseInt(cat?.score?.quality_score_average))}
                </span>{" "}
                /100 Quality
              </p>
            </div>
            <div className="card-score">
              <p>
                <span>
                  {Math.round(parseInt(cat?.score?.fluency_score_average))}
                </span>{" "}
                /100 Fluency
              </p>
            </div>
            <div className="card-score">
              <p>
                <span>
                  {Math.round(parseInt(cat?.score?.pte_score_average))}
                </span>{" "}
                /90 PTE
              </p>
            </div>
            <div className="card-score">
              <p>
                <span>
                  {Math.round(parseInt(cat?.score?.ielts_score_average))}
                </span>{" "}
                /9 IELTS
              </p>
            </div>
          </div>
        </>
      ) : (
        <>
          <p>{`${finishText} / ${cat?.text?.length}`}</p>
          <div className="assessment-card-button">
            <ButtonGeneric
              text={
                finishText === cat?.text?.length && finishText !== 0
                  ? "Show result"
                  : finishText > 0
                  ? "Continue"
                  : "Start"
              }
              event={
                finishText === cat?.text?.length && finishText !== 0
                  ? handleShowScore
                  : () => handleShowAssesment(index, cat?.category)
              }
              size="small"
              hasOutline={true}
              loading={showScore}
              disabled={showScore}
            />
          </div>
        </>
      )}
    </div>
  );
};

export default React.memo(CategoryCard);
