import React, {
  useEffect,
  useState,
  useContext,
  useCallback,
  useRef,
  Suspense,
  lazy,
} from "react";
import Swal from "sweetalert2";
import Moment from "react-moment";
import { Switch, Route, useRouteMatch, useHistory } from "react-router-dom";

import { getUserData } from "../../utils/Common";
import { getTextSpeech, getUserTextSpeechOutput } from "../../api";
import InnerHeader from "../Headers/InnerHeader";
import {
  HeadingTitle,
  SectionLoader,
  PageLoader,
  ButtonGeneric,
} from "../_reusables";
import { DataContext, AssesmentContext } from "../../context";
import CategoryCard from "./CategoryCard";
import MicTest from "./MicTest";
// import AssesmentExam from "./AssesmentExam";

import "./_LanguageAssesment.scss";

const AssesmentExam = lazy(() => import("./AssesmentExam"));

const LanguageAssesment = () => {
  const { setLanguageLocation } = useContext(DataContext);
  const [categories, setCategories] = useState([]);
  const [loading, setLoading] = useState(false);
  const [completedCount, setCompletedCount] = useState(0);
  const [currentAssesment, setCurrentAssesment] = useState(null);
  const [isTestMic, setTestMic] = useState(false);
  const [render, setRender] = useState(false);
  // const [userTextOutput, setUserTextOutput] = useState([]);
  const userTextOutput = useRef([]);

  const fromLocal = getUserData();

  let { path, url } = useRouteMatch();
  const history = useHistory();

  const handleShowAssesment = (index, cat) => {
    if (categories[index]?.text.length === 0) {
      Swal.fire({
        title: "",
        text: "No available text speech tests.",
        icon: "info",
        confirmButtonText: "Close",
        confirmButtonColor: "#55428A",
      });
      return;
    }

    setCurrentAssesment(categories[index]);
    console.log(userTextOutput.current.length);
    if (userTextOutput.current.length > 0) {
      const current = userTextOutput.current.find(
        (item) => item.category === cat
      );

      console.log(current);

      if (current) {
        const id = categories[index].text[current.text.length]?.id;

        console.log(categories[index]);

        console.log(id);

        history.push(
          `${url}/${
            categories[index].category === "Pronounciation"
              ? "Pronunciation"
              : categories[index].category
          }/${id}`
        );
      } else {
        history.push(
          `${url}/${
            categories[index].category === "Pronounciation"
              ? "Pronunciation"
              : categories[index].category
          }/${categories[index]?.text[0]?.id}`
        );
      }
    } else {
      console.log("yes");
      history.push(
        `${url}/${
          categories[index].category === "Pronounciation"
            ? "Pronunciation"
            : categories[index].category
        }/${categories[index]?.text[0]?.id}`
      );
    }
  };

  useEffect(() => {
    // if (process.env.REACT_APP_HIDE_LANGUAGE_ASSESSMENT === 1) {
    //   history.push(fromLocal?.next_link);
    //   return;
    // }

    const controller = new AbortController();
    setLoading(true);
    getTextSpeech(fromLocal.token, controller)
      .then((res) => {
        if (res.data.success) {
          const speeches = res.data.data;

          getUserTextSpeechOutput(fromLocal.token, controller).then((res) => {
            if (res.data.success) {
              const results = res.data.data;
              let newCategories = speeches;
              let newUserTextOutput = [];
              let totalCompletedCount = 0;

              console.log(results);

              if (results) {
                speeches.forEach((speech, index) => {
                  for (const key in results) {
                    if (
                      results[key].category === speech.category &&
                      results[key].text.length <= speech.text.length
                    ) {
                      console.log(results[key]);
                      console.log(speech);
                      if (results.hasOwnProperty(key)) {
                        const {
                          text,
                          fluency_score_average,
                          ielts_score_average,
                          pte_score_average,
                          quality_score_average,
                          category,
                        } = results[key];

                        const score = {
                          fluency_score_average,
                          ielts_score_average,
                          pte_score_average,
                          quality_score_average,
                        };

                        const cat = {
                          ...speech,
                          score: fluency_score_average ? score : speech.score,
                          text: speech.text.map((textItem, i) => ({
                            ...textItem,
                            ...text[i],
                          })),
                        };

                        if (results[key].text.length === speech.text.length) {
                          totalCompletedCount += 1;
                        }

                        newCategories.splice(index, 1, cat);
                        newUserTextOutput.push({ category, text });
                      }
                    }
                  }
                });

                setCategories(newCategories);
                setCompletedCount(totalCompletedCount);
                userTextOutput.current = newUserTextOutput;
              } else {
                setCategories(speeches);
              }
              setLanguageLocation(true);
              setLoading(false);
            }
          });
        }
      })
      .catch((err) => {
        setLoading(false);
        if (err.response.status === 401) {
          Swal.fire({
            title: "Error!",
            text: err.response.data.message,
            icon: "error",
            confirmButtonText: "Close",
            confirmButtonColor: "#55428A",
          });
          return;
        }

        if (err.response.status === 500) {
          console.log(err.response.data.message);
        }
      });

    return () => controller.abort();
  }, [render]);
  return (
    <AssesmentContext.Provider
      value={{
        categories,
        setCategories,
        currentAssesment,
        setCurrentAssesment,
        setCompletedCount,
        userTextOutput,
        setRender,
      }}
    >
      {/* <InnerHeader /> */}
      {isTestMic && <MicTest setTestMic={setTestMic} />}
      <div className="container">
        <div className="white-box-container">
          <HeadingTitle text="Online language assessment" />
          {!loading ? (
            <>
              <div className="assesment-user-detail-container">
                <div className="assesment-user-detail">
                  <span>Date and time:</span>
                  <span>
                    <Moment date={new Date()} format="MMMM D, YYYY, h:mm a" />
                  </span>
                </div>
                <div className="assesment-user-detail">
                  <span>Applicant name:</span>
                  <span>{`${fromLocal?.first_name} ${fromLocal?.last_name}`}</span>
                </div>
                <div className="assesment-user-detail">
                  <span>Email address:</span>
                  <span>{fromLocal?.email}</span>
                </div>
              </div>
              <div className="assesment-instruction-container">
                <p>Assessment rules and reminder:</p>
                <ul>
                  <li>This is just a one time online assessment.</li>
                  <li>Please ensure that your connection is stable.</li>
                  <li>Please make sure that your microphone is working.</li>
                </ul>
              </div>
              <div style={{ width: 150, marginTop: 30 }}>
                <ButtonGeneric
                  text="Test microphone"
                  size="small"
                  // hasOutline={true}
                  event={() => setTestMic(true)}
                />
              </div>
              <div className="assesment-text-list-container">
                <Suspense
                  fallback={
                    <PageLoader
                      color1={"#463672"}
                      color2={"#fff"}
                      containerHeight={"100%"}
                    />
                  }
                >
                  <Switch>
                    <Route exact path={`${path}`}>
                      <>
                        <div className="assesment-completed-count">
                          <span>{`${completedCount} of ${categories?.length} categories completed`}</span>
                        </div>
                        <div className="assessment-text-list-grid">
                          {categories.map((item, i) => (
                            <CategoryCard
                              key={item.id}
                              category={item}
                              index={i}
                              handleShowAssesment={handleShowAssesment}
                              userTextOutput={userTextOutput}
                              setCategories={setCategories}
                              categories={categories}
                            />
                          ))}
                        </div>
                      </>
                    </Route>

                    <Route
                      path={`${path}/:category/:textId`}
                      component={AssesmentExam}
                    />
                  </Switch>
                </Suspense>
              </div>
            </>
          ) : (
            <SectionLoader color1={"#463672"} color2={"#fff"} />
          )}
        </div>
      </div>
    </AssesmentContext.Provider>
  );
};

export default LanguageAssesment;
