import React, { useState, useEffect, useRef } from "react";
import AudioReactRecorder, { RecordState } from "audio-react-recorder";
import { Bars } from "react-loader-spinner";

import {
  micOnIcon,
  micOffIcon,
  StopIcon,
  MicrophoneIcon,
} from "../../utils/SvgIcons";
import { ButtonGeneric } from "../_reusables";
import "./_MicTest.scss";

const MicTest = ({ setTestMic }) => {
  const [isMicSet, setMic] = useState(null);
  const [isMicTest, setMicTest] = useState(false);
  const [micError, setMicError] = useState("");
  const [isNext, setNext] = useState(false);
  const [recordData, setRecordData] = useState(null);
  const [isRecording, setRecording] = useState(null);
  const [startRec, setStartRec] = useState(false);
  const [microphone, setMicrophone] = useState(null);

  const handleDoneNext = () => {
    if (!isNext) {
      setNext(true);
    } else {
      microphone.forEach(function (track) {
        track.stop();
      });
      setNext(false);
      setTestMic(false);
      setMicrophone(null);
    }
  };

  const handleRecordStart = () => {
    setRecordData(null);
    setRecording(RecordState.START);
  };
  const handleRecordStop = () => setRecording(RecordState.STOP);
  // const handlePlay = () => audioRef.current.play();

  const handleOnStop = (audioData) => {
    setRecordData(audioData);
  };

  useEffect(() => {
    if (!isNext) {
      navigator.mediaDevices
        .getUserMedia({ audio: true })
        .then((stream) => {
          const mic = stream.getAudioTracks();
          if (mic[0].enabled) {
            setMicrophone(mic);
            setMic(true);
            setMicTest(true);
          }
        })
        .catch((err) => {
          setMic(false);
          setMicTest(false);

          if (
            err.name === "NotAllowedError" ||
            err.name === "PermissionDeniedError"
          ) {
            setMicError(
              "Make sure that the granted access permission is enabled."
            );
            return;
          }

          if (
            err.name === "NotReadableError" ||
            err.name === "TrackStartError"
          ) {
            setMicError("Microphone is already in use in other application.");
            return;
          }

          if (
            err.name === "NotFoundError" ||
            err.name === "DevicesNotFoundError"
          ) {
            setMicError("No microphone connected on your machine.");
            return;
          }
        });
    }
  }, []);

  return (
    <div className="assesment-mic-test-main">
      <div className="assesment-mic-test-sub">
        {!isNext ? (
          <div className="mic-container">
            {isMicTest ? (
              <div className="test-container">
                <div>
                  {isMicSet ? micOnIcon("#7964b5") : micOffIcon("#aeadb4")}
                </div>
                <p className={`${!isMicSet ? "hasError" : ""}`}>Microphone</p>
                {!isMicSet && <p className="error-test">{micError}</p>}
                {isMicSet && (
                  <p className="success-test">
                    No issue with your current device.
                  </p>
                )}
              </div>
            ) : (
              <div className="mic-loader">
                <div>
                  <Bars height="40" width="80" color="#7964b5" />
                </div>
                <p>Checking microphone...</p>
              </div>
            )}
          </div>
        ) : (
          <div className="mic-test-record-container">
            {!startRec && (
              <>
                <p>Record an audio to hear how you sound.</p>
                <ButtonGeneric
                  text="Record an audio clip"
                  size="small"
                  event={() => setStartRec(true)}
                />
              </>
            )}

            {startRec && (
              <>
                <p>
                  Say something, no one else will be able to play this and it
                  will not be saved
                </p>
                <button
                  className="mic-record-btn"
                  onClick={
                    isRecording === "start"
                      ? handleRecordStop
                      : handleRecordStart
                  }
                >
                  {isRecording === "start"
                    ? StopIcon("#f53a29")
                    : MicrophoneIcon("#55428a")}
                </button>

                {recordData && (
                  <audio
                    style={{ display: "block", margin: "30px auto 0" }}
                    src={recordData.url}
                    type={recordData.type}
                    controls
                    controlsList="nodownload"
                  />
                )}

                <AudioReactRecorder
                  state={isRecording}
                  type="audio/mp3"
                  backgroundColor="#fff"
                  foregroundColor="#55428a"
                  onStop={handleOnStop}
                  canvasHeight={100}
                />
              </>
            )}
          </div>
        )}
        <div className="mic-test-btn-container">
          <ButtonGeneric
            text={isNext ? "Done" : "Next"}
            size="extraSmall"
            hasOutline={true}
            event={handleDoneNext}
          />
        </div>
      </div>
    </div>
  );
};

export default React.memo(MicTest);
